#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdarg.h>
#include <stdint.h>
#include <sysexits.h>
#include <unistd.h>

#include <string.h>
#include <ctype.h>
#include <errno.h>

#include <openssl/evp.h>

#define	ENC_MAX_BUFSIZ		0xffff
#define ENC_AES_IV_SIZ		16

static int32_t		input_file(uint8_t **, char *, int32_t *);

int
main(int argc, char **argv)
{
	int opt, error = EX_OK, decrypt = 0, inl, outl, keyl;
	FILE *out;
	uint8_t *inb, *outb, *keyb, iv[ENC_AES_IV_SIZ];
	EVP_CIPHER_CTX ctx;

	out = NULL;
	inb = outb = keyb = NULL;
	while ((opt = getopt(argc, argv, "di:k:o:")) != EOF) {
		switch (opt) {
		case 'd':
			decrypt = 1;
			break;
		case 'i':
			if (optarg == NULL ||
			    input_file(&inb, optarg, &inl) != 0)
				error = EX_OSERR;
			break;
		case 'k':
			if (optarg == NULL ||
			    input_file(&keyb, optarg, &keyl) != 0)
				error = EX_OSERR;
			break;
		case 'o':
			if (optarg == NULL ||
			    (out = fopen(optarg, "rw")) == NULL)
				error = EX_OSERR;
			break;
		}
	}

	if (error != EX_OK)
		goto cleanup;

	memcpy(iv, keyb, ENC_AES_IV_SIZ);
	if (decrypt == 0) {
		if (EVP_EncryptInit(&ctx, EVP_aes_128_cfb128(), keyb, iv) != 1) {
			error = EX_DATAERR;
			goto cleanup;
		}
		if (EVP_EncryptUpdate(&ctx, outb, &outl, inb, inl) != 1 ||
		    EVP_EncryptFinal(&ctx, outb + outl, &outl) != 1)
			error = EX_DATAERR;
	} else {
		if (EVP_DecryptInit(&ctx, EVP_aes_128_cfb128(), keyb, iv) != 1 ||
		    EVP_CIPHER_CTX_set_padding(&ctx, 0) != 1) {
			error = EX_DATAERR;
			goto cleanup;
		}
		if (EVP_DecryptUpdate(&ctx, outb, &outl, inb, inl) != 1 ||
		    EVP_DecryptFinal(&ctx, outb + outl, &outl) != 1)
			error = EX_DATAERR;
	}

	EVP_CIPHER_CTX_cleanup(&ctx);
	/* XXX: save result */
cleanup:
	if (out != NULL)
		fclose(out);
	if (inb != NULL)
		free(inb);
	if (outb != NULL)
		free(outb);
	if (keyb != NULL)
		free(keyb);
		
	exit(error);
}

static int32_t
input_file(uint8_t **buf, char *fn, int32_t *sz)
{
	return (0);
}
