/*-
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 69):
 * <syrinx@FreeBSD.org> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   -Shteryana Shopova
 * ----------------------------------------------------------------------------
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <sys/un.h>
#include <sys/uio.h>
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#ifndef LINUX
#include <string.h>
#endif
#include <stdarg.h>
#include <ctype.h>
#include <err.h>
#include <errno.h>
#include <sysexits.h>
#include <syslog.h>
#include <unistd.h>
#include <signal.h>
#include <dlfcn.h>
#include <inttypes.h>

#include <arpa/inet.h>
#include <netinet/in.h>

#ifdef LINUX
#include <bsd/string.h>
#endif
#include <openssl/err.h>
#include <openssl/rsa.h>
#include <openssl/ssl.h>

#define	PSERVER_PORT		2121
#define	PSERVER_HOST		"localhost"
#define	PSERVER_MAXQUEUE	10

static int	work = 1;
/* static void	usage(char *); */
static int	process_client(int, struct sockaddr_in *, int);
static int	init_signals(void);
static void	onint(int);

int
main(int argc, char **argv)
{
	int sock = -1, csock = -1, val = 1, error;
	socklen_t clen;
	pid_t cpid;
	struct in_addr saddr = { INADDR_ANY };
	struct sockaddr_in sin, cin;
	/* BIO *sbio = NULL; */

	if ((error = init_signals()) != EX_OK)
		err(error, "init signals failed");

	if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
		err(EX_OSERR, "socket failed");
	
	memset(&sin, 0, sizeof(sin));
	sin.sin_addr.s_addr = saddr.s_addr;
	sin.sin_family = AF_INET;
	sin.sin_port = htons(PSERVER_PORT);
	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &val,sizeof(val)) == -1)
		err(EX_OSERR, "setsockopt failed");

	if (bind(sock,(struct sockaddr *)&sin, sizeof(sin)) == -1)
		err(EX_OSERR, "bind failed");

	if (listen(sock, PSERVER_MAXQUEUE) == -1)
		err(EX_OSERR, "listen failed");

	for ( ; work != 0 ; ) {
		memset(&cin, 0, sizeof(cin));
		clen = sizeof(cin);
		if ((csock = accept(sock, (struct sockaddr *) &cin, &clen)) ==
		    -1) {
			warn("accept failed");
			continue;
		}
		switch ((cpid = fork())) {
		case -1:
			warn("fork failed");
			break;
		case 0:
			if ((error = process_client(csock, &cin, clen)) != EX_OK)
				warn("failed to process client request");
			exit(error);
		default:
			close(csock);
			break;
		}
	}

	close(sock);
	exit(0);
}

static int
process_client(int csock, struct sockaddr_in *cin, int clen)
{
	BIO *sbio = NULL;

	warn("Processing connection from client 0x%x",
	    ntohl(cin->sin_addr.s_addr));

	return (EX_OK);
}

static int
init_signals(void)
{
	struct sigaction sa;

	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = onint;
	sa.sa_flags = 0;
	sigemptyset(&sa.sa_mask);
	if (sigaction(SIGINT, &sa, NULL)) {
		warn("sigaction(%d)", SIGINT);
		return (EX_OSERR);
	}

	return (EX_OK);
}

static void
onint(int s __unused)
{
	work = 0;
}
